@@ -79,6 +79,11 @@ class StructureScore(BaseEstimator):
         """A (log) prior distribution over models. Currently unused (= uniform)."""
         return 0
 
+    def structure_prior_ratio(self, operation):
+        """Return the log ratio of the prior probabilities for a given proposed change to the DAG.
+        Currently unused (=uniform)."""
+        return 0
+
 
 class K2Score(StructureScore):
     def __init__(self, data, **kwargs):
@@ -182,6 +187,9 @@ class BDeuScore(StructureScore):
         self.equivalent_sample_size = equivalent_sample_size
         super(BDeuScore, self).__init__(data, **kwargs)
 
+    def get_number_of_parent_states(self, state_counts):
+        return float(state_counts.shape[1])
+
     def local_score(self, variable, parents):
         'Computes a score that measures how much a \
         given variable is "influenced" by a given list of potential parents.'
@@ -189,14 +197,12 @@ class BDeuScore(StructureScore):
         var_states = self.state_names[variable]
         var_cardinality = len(var_states)
         state_counts = self.state_counts(variable, parents)
-        num_parents_states = float(state_counts.shape[1])
+        num_parents_states = self.get_number_of_parent_states(state_counts)
 
         counts = np.asarray(state_counts)
         log_gamma_counts = np.zeros_like(counts, dtype=np.float_)
-
         alpha = self.equivalent_sample_size / num_parents_states
         beta = self.equivalent_sample_size / counts.size
-
         # Compute log(gamma(counts + beta))
         gammaln(counts + beta, out=log_gamma_counts)
 
@@ -210,7 +216,71 @@ class BDeuScore(StructureScore):
             + num_parents_states * lgamma(alpha)
             - counts.size * lgamma(beta)
         )
+        return score
 
+
+class BDsScore(BDeuScore):
+    def __init__(self, data, equivalent_sample_size=10, **kwargs):
+        """
+        Class for Bayesian structure scoring for BayesianModels with Dirichlet priors.
+        The BDs score is the result of setting all Dirichlet hyperparameters/pseudo_counts to
+        `equivalent_sample_size/modified_variable_cardinality` where for the modified_variable_cardinality
+        only the number of parent configurations where there were observed variable counts are considered.
+        The `score`-method measures how well a model is able to describe the given data set.
+
+        Parameters
+        ----------
+        data: pandas DataFrame object
+            datafame object where each column represents one variable.
+            (If some values in the data are missing the data cells should be set to `numpy.NaN`.
+            Note that pandas converts each column containing `numpy.NaN`s to dtype `float`.)
+
+        equivalent_sample_size: int (default: 10)
+            The equivalent/imaginary sample size (of uniform pseudo samples) for the dirichlet
+            hyperparameters.
+            The score is sensitive to this value, runs with different values might be useful.
+
+        state_names: dict (optional)
+            A dict indicating, for each variable, the discrete set of states (or values)
+            that the variable can take. If unspecified, the observed values in the data set
+            are taken to be the only possible states.
+
+        complete_samples_only: bool (optional, default `True`)
+            Specifies how to deal with missing data, if present. If set to `True` all rows
+            that contain `np.Nan` somewhere are ignored. If `False` then, for each variable,
+            every row where neither the variable nor its parents are `np.NaN` is used.
+            This sets the behavior of the `state_count`-method.
+
+        References
+        ---------
+        [1] Scutari, Marco. An Empirical-Bayes Score for Discrete Bayesian Networks.
+        Journal of Machine Learning Research, 2016, pp. 438–48
+
+        """
+        super(BDsScore, self).__init__(data, equivalent_sample_size, **kwargs)
+
+    def get_number_of_parent_states(self, state_counts):
+        return float(len(np.where(state_counts.sum(axis=0) > 0)[0]))
+
+    def structure_prior_ratio(self, operation):
+        """Return the log ratio of the prior probabilities for a given proposed change to
+        the DAG.
+        """
+        if operation == "+":
+            return -log(2.0)
+        if operation == "-":
+            return log(2.0)
+        return 0
+
+    def structure_prior(self, model):
+        """
+        Implements the marginal uniform prior for the graph structure where each arc
+        is independent with the probability of an arc for any two nodes in either direction
+        is 1/4 and the probability of no arc between any two nodes is 1/2."""
+        nedges = float(len(model.edges()))
+        nnodes = float(len(model.nodes()))
+        possible_edges = nnodes * (nnodes - 1) / 2.0
+        score = -(nedges + possible_edges) * log(2.0)
         return score
 
 
