@@ -1,10 +1,7 @@
 #!/usr/bin/env python
-from warnings import warn
 from functools import lru_cache
 
-import numpy as np
 import pandas as pd
-from scipy.stats import chisquare
 
 from pgmpy.utils.decorators import convert_args_tuple
 
@@ -17,7 +14,6 @@ class BaseEstimator(object):
 
         Parameters
         ----------
-
         data: pandas DataFrame object datafame object where each column represents one variable.
             (If some values in the data are missing the data cells should be set to `numpy.NaN`.
             Note that pandas converts each column containing `numpy.NaN`s to dtype `float`.)
@@ -61,13 +57,15 @@ class BaseEstimator(object):
                         self.state_names[var] = self._collect_state_names(var)
 
     def _collect_state_names(self, variable):
-        "Return a list of states that the variable takes in the data"
+        "Return a list of states that the variable takes in the data."
         states = sorted(list(self.data.loc[:, variable].dropna().unique()))
         return states
 
     @convert_args_tuple
     @lru_cache(maxsize=2048)
-    def state_counts(self, variable, parents=[], complete_samples_only=None):
+    def state_counts(
+        self, variable, parents=[], complete_samples_only=None, weighted=False
+    ):
         """
         Return counts how often each state of 'variable' occurred in the data.
         If a list of parents is provided, counting is done conditionally
@@ -88,6 +86,10 @@ class BaseEstimator(object):
             every row where neither the variable nor its parents are `np.NaN` is used.
             Desired default behavior can be passed to the class constructor.
 
+        weighted: bool
+            If True, data must have a `_weight` column specifying the weight of the
+            datapoint (row). If False, each datapoint has a weight of `1`.
+
         Returns
         -------
         state_counts: pandas.DataFrame
@@ -129,9 +131,16 @@ class BaseEstimator(object):
             else self.data.dropna(subset=[variable] + parents)
         )
 
+        if weighted and ("_weight" not in self.data.columns):
+            raise ValueError("data must contain a `_weight` column if weighted=True")
+
         if not parents:
             # count how often each state of 'variable' occured
-            state_count_data = data.loc[:, variable].value_counts()
+            if weighted:
+                state_count_data = data.groupby([variable]).sum()["_weight"]
+            else:
+                state_count_data = data.loc[:, variable].value_counts()
+
             state_counts = (
                 state_count_data.reindex(self.state_names[variable])
                 .fillna(0)
@@ -141,9 +150,16 @@ class BaseEstimator(object):
         else:
             parents_states = [self.state_names[parent] for parent in parents]
             # count how often each state of 'variable' occured, conditional on parents' states
-            state_count_data = (
-                data.groupby([variable] + parents).size().unstack(parents)
-            )
+            if weighted:
+                state_count_data = (
+                    data.groupby([variable] + parents).sum()["_weight"].unstack(parents)
+                )
+
+            else:
+                state_count_data = (
+                    data.groupby([variable] + parents).size().unstack(parents)
+                )
+
             if not isinstance(state_count_data.columns, pd.MultiIndex):
                 state_count_data.columns = pd.MultiIndex.from_arrays(
                     [state_count_data.columns]
@@ -189,7 +205,7 @@ class ParameterEstimator(BaseEstimator):
             This sets the behavior of the `state_count`-method.
         """
 
-        if not set(model.nodes()) <= set(data.columns.values):
+        if not (set(model.nodes()) - model.latents) <= set(data.columns.values):
             raise ValueError(
                 "variable names of the model must be identical to column names in data"
             )
@@ -197,7 +213,7 @@ class ParameterEstimator(BaseEstimator):
 
         super(ParameterEstimator, self).__init__(data, **kwargs)
 
-    def state_counts(self, variable, **kwargs):
+    def state_counts(self, variable, weighted=False, **kwargs):
         """
         Return counts how often each state of 'variable' occurred in the data.
         If the variable has parents, counting is done conditionally
@@ -243,12 +259,9 @@ class ParameterEstimator(BaseEstimator):
 
         parents = sorted(self.model.get_parents(variable))
         return super(ParameterEstimator, self).state_counts(
-            variable, parents=parents, **kwargs
+            variable, parents=parents, weighted=weighted, **kwargs
         )
 
-    def get_parameters(self):
-        pass
-
 
 class StructureEstimator(BaseEstimator):
     def __init__(self, data=None, independencies=None, **kwargs):
