@@ -11,15 +11,16 @@ from pgmpy.models import BayesianModel
 class TestMmhcEstimator(unittest.TestCase):
     def setUp(self):
         self.data1 = pd.DataFrame(
-            np.random.randint(0, 2, size=(15000, 3)), columns=list("XYZ")
+            np.random.randint(0, 2, size=(int(1e5), 3)), columns=list("XYZ")
         )
         self.data1["sum"] = self.data1.sum(axis=1)
         self.est1 = MmhcEstimator(self.data1)
 
-    @unittest.skip("currently disabled due to non-determenism")
     def test_estimate(self):
+        dag1 = self.est1.estimate()
+        self.assertTrue(len(dag1.edges()) > 1)
         self.assertTrue(
-            set(self.est1.estimate().edges()).issubset(
+            set(dag1.edges()).issubset(
                 set(
                     [
                         ("X", "sum"),
@@ -28,12 +29,20 @@ class TestMmhcEstimator(unittest.TestCase):
                         ("sum", "X"),
                         ("sum", "Y"),
                         ("sum", "Z"),
+                        ("X", "Y"),
+                        ("X", "Z"),
+                        ("Y", "Z"),
+                        ("Y", "X"),
+                        ("Z", "X"),
+                        ("Z", "Y"),
                     ]
                 )
             )
         )
+        dag2 = self.est1.estimate(significance_level=0.001)
+        self.assertTrue(len(dag2.edges()) > 1)
         self.assertTrue(
-            set(self.est1.estimate(significance_level=0.001).edges()).issubset(
+            set(dag2.edges()).issubset(
                 set(
                     [
                         ("X", "sum"),
@@ -42,6 +51,12 @@ class TestMmhcEstimator(unittest.TestCase):
                         ("sum", "X"),
                         ("sum", "Y"),
                         ("sum", "Z"),
+                        ("X", "Y"),
+                        ("X", "Z"),
+                        ("Y", "Z"),
+                        ("Y", "X"),
+                        ("Z", "X"),
+                        ("Z", "Y"),
                     ]
                 )
             )
